@@ -172,6 +172,7 @@ totalAnovaRandLsmeans <- function(model, ddf="Satterthwaite", type = 3, alpha.ra
           
           # calculate asymptotic covariance matrix A
           h  <-  hessian(function(x) Dev(rho,x), rho$param$vec.matr)
+          
           rho$A <- 2*solve(h)
           #rho$A <- 2*ginv(h)
           
@@ -191,8 +192,8 @@ totalAnovaRandLsmeans <- function(model, ddf="Satterthwaite", type = 3, alpha.ra
     rho <- rhoInit(model)     
     
     # calculate asymptotic covariance matrix A
-    #h  <-  hessian(function(x) Dev(rho,x), rho$param$vec.matr)
-    h  <-  myhess(function(x) Dev(rho,x), rho$param$vec.matr)
+    h  <-  hessian(function(x) Dev(rho,x), rho$param$vec.matr)
+    #h  <-  myhess(function(x) Dev(rho,x), rho$param$vec.matr)
     rho$A  <-  2*solve(h)
 
     
@@ -246,17 +247,23 @@ totalAnovaRandLsmeans <- function(model, ddf="Satterthwaite", type = 3, alpha.ra
     X.design <- X.design.list$X.design
     names.design.withLevels <- X.design.list$names.design.withLevels
     
-
-    
     
     #save full coefficients in rho
-    nums.dummy.coefs <- getNumsDummyCoefs(model, data, l)
-    rho$nums.zeroCoefs <- nums.dummy.coefs$nums.zeroCoefs
-    rho$nums.Coefs <- nums.dummy.coefs$nums.Coefs
+    ## old code, worked with dummy.coef
+    #nums.dummy.coefs <- getNumsDummyCoefs(model, data, l)
+    #rho$nums.zeroCoefs <- nums.dummy.coefs$nums.zeroCoefs
+    #rho$nums.Coefs <- nums.dummy.coefs$nums.Coefs
+    #fullCoefs <- rep(0, ncol(X.design))
+    #fullCoefs[rho$nums.Coefs] <- rho$fixEffs
+    #fullCoefs <- setNames(fullCoefs, names.design.withLevels) 
+    ###new code with X.design matrix
     fullCoefs <- rep(0, ncol(X.design))
-    fullCoefs[rho$nums.Coefs] <- rho$fixEffs
-    
-    
+    fullCoefs <- setNames(fullCoefs, names.design.withLevels) 
+    names(fullCoefs)[1] <- "(Intercept)"
+    fullCoefs[names(rho$fixEffs)] <- rho$fixEffs
+    rho$nums.Coefs <- which(names(fullCoefs) %in% names(rho$fixEffs))
+      #unique(c(which(names(fullCoefs)=="(Intercept)"),which(names(fullCoefs) %in% names(rho$fixEffs))))
+    rho$nums.Coefs <- setNames(rho$nums.Coefs, names(fullCoefs[rho$nums.Coefs]))
     #define the terms that are to be tested
     test.terms <- attr(terms(model),"term.labels")
     
@@ -274,6 +281,7 @@ totalAnovaRandLsmeans <- function(model, ddf="Satterthwaite", type = 3, alpha.ra
     
     
     # calculate type 1 hypothesis matrices for each term
+    # TODO: fix bug with noint sens1 + Homesize
     if( type==1 )
     {
       #X <- model.matrix(model)
@@ -313,7 +321,6 @@ totalAnovaRandLsmeans <- function(model, ddf="Satterthwaite", type = 3, alpha.ra
     
   }
   
-  
   #convert anova table to data frame
   anova.table <- as.data.frame(anova.table)
   anova.table$NumDF <- as.integer(anova.table$NumDF)
@@ -350,6 +357,9 @@ totalAnovaRandLsmeans <- function(model, ddf="Satterthwaite", type = 3, alpha.ra
   tsummary <- calculateTtest(rho, diag(rep(1,length(rho$fixEffs))), length(rho$fixEffs), method.grad)
   result$ttest <- list(df=tsummary[,"df"], tvalue=tsummary[,"t value"], tpvalue=tsummary[,"p-value"])
   
+  #format anova.table and random.table according to elim.num column
+  result$anova.table <- formatElimNumTable(result$anova.table) 
+  result$rand.table <- formatElimNumTable(result$rand.table) 
   
   #update final model
   mf.final <- update.formula(formula(model),formula(model))
@@ -368,7 +378,7 @@ totalAnovaRandLsmeans <- function(model, ddf="Satterthwaite", type = 3, alpha.ra
 
 
 
-step <- function(model, ddf="Satterthwaite", type=3, alpha.random = 0.1, alpha.fixed = 0.05, reduce.fixed = TRUE, reduce.random = TRUE, lsmeans.calc=TRUE, difflsmeans.calc=TRUE, test.effs=NULL, method.grad="simple",...)
+step <- function(model, ddf="Satterthwaite", type=3, alpha.random = 0.1, alpha.fixed = 0.05, reduce.fixed = TRUE, reduce.random = TRUE, lsmeans.calc=TRUE, difflsmeans.calc=TRUE, test.effs=NULL, method.grad="simple", ...)
 {  
   result <- totalAnovaRandLsmeans(model=model, ddf=ddf , type=type,  alpha.random=alpha.random, alpha.fixed=alpha.fixed, reduce.fixed=reduce.fixed, reduce.random=reduce.random, lsmeans.calc=lsmeans.calc, difflsmeans.calc=difflsmeans.calc, isTotal=TRUE, isTtest=FALSE, test.effs=test.effs, method.grad=method.grad)
   class(result) <- "step"
@@ -389,8 +399,11 @@ print.step <- function(x, ...)
   
   if(!is.null(x$rand.table))
   {
-    cat("\nRandom effects:\n")  
-    printCoefmat(x$rand.table, digits=3 , dig.tst=1  ,tst.ind=c(which(colnames(x$rand.table)=="Chi.DF"),which(colnames(x$rand.table)=="elim.num")), P.values=TRUE, has.Pvalue=TRUE)
+    cat("\nRandom effects:\n") 
+    x$rand.table[,"p.value"] <- format.pval(x$rand.table[,"p.value"], digits=4, eps=1e-7)
+    x$rand.table[,"Chi.sq"] <- round(x$rand.table[,"Chi.sq"],2)
+    print(x$rand.table)
+    #printCoefmat(x$rand.table, digits=3 , dig.tst=1  ,tst.ind=which(colnames(x$rand.table)=="Chi.DF"), P.values=TRUE, has.Pvalue=TRUE, na.print = "KEEP")
   }
   
   if(nrow(x$anova.table) != 0)
@@ -408,7 +421,11 @@ print.step <- function(x, ...)
     else
     {
       cat("\nFixed effects:\n")
-      printCoefmat(x$anova.table, dig.tst=3, tst.ind=3, cs.ind=3, digits=3 ,P.values = TRUE, has.Pvalue=TRUE)
+      x$anova.table[,"Pr(>F)"] <- format.pval(x$anova.table[,"Pr(>F)"], digits=4, eps=1e-7)
+      x$anova.table[,c("Sum Sq","Mean Sq", "F.value")] <- round(x$anova.table[,c("Sum Sq","Mean Sq", "F.value")],4)
+      x$anova.table[,"DenDF"] <- round(x$anova.table[,"DenDF"],2)
+      print(x$anova.table) 
+       #printCoefmat(x$anova.table, dig.tst=3, tst.ind=3, cs.ind=3, digits=3 ,P.values = TRUE, has.Pvalue=TRUE)
       if(!is.null(x$lsmeans.table))
       {
         cat("\nLeast squares means:\n")
@@ -425,7 +442,7 @@ print.step <- function(x, ...)
   else
     print(x$anova.table)
   cat("\nFinal model:\n")
-  print(x$model@call)  
+  print(x$model@call) 
 }
 
 
@@ -512,7 +529,7 @@ setMethod("anova", signature(object="merModLmerTest"),
                       
                       
                       table <- an.table
-                    attr(table, "heading") <- paste("Analysis of Variance Table of type", type ,"with ", ddf, " approximation for degrees of freedom")
+                    attr(table, "heading") <- paste("Analysis of Variance Table of type", type ," with ", ddf, "\napproximation for degrees of freedom")
                   }
                   
                   
@@ -533,11 +550,12 @@ setMethod("summary", signature(object = "merModLmerTest"),
             if(!is.null(ddf) && ddf=="lme4") return(cl)
             else
             {
-             
-                t.pval <- tryCatch( {totalAnovaRandLsmeans(model=object, ddf="Satterthwaite", isTtest=TRUE)$ttest$tpvalue}, error = function(e) { NULL })
-                coefs.satt <- cbind(cl$coefficients, t.pval) 
+              tsum <- tryCatch( {totalAnovaRandLsmeans(model=object, ddf="Satterthwaite", isTtest=TRUE)$ttest}, error = function(e) { NULL })
+              coefs.satt <- cbind(cl$coefficients[,1:2], tsum$df, tsum$tvalue, tsum$tpvalue)
+               # t.pval <- tryCatch( {totalAnovaRandLsmeans(model=object, ddf="Satterthwaite", isTtest=TRUE)$ttest$tpvalue}, error = function(e) { NULL })
+               # coefs.satt <- cbind(cl$coefficients, t.pval) 
                 cl$coefficients <- coefs.satt
-                colnames(cl$coefficients)[4] <- "Pr(>|t|)"              
+                colnames(cl$coefficients)[3:5] <- c("df","t value","Pr(>|t|)")              
             }   
             
             return(cl)
